VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsRiderGrid"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Dim arGrid(12, 12) As New clsRBlock
Dim arBSelected(12, 12) As Boolean
Dim objColor As New clsPallette
Dim bDirty As Boolean
Dim bSelectOn As Boolean

Public Function Width() As Integer
    Width = 12
End Function

Public Function Height() As Integer
    Height = 12
End Function

Public Function IsSelected() As Boolean
    IsSelected = bSelectOn
End Function

Public Function IsChanged() As Boolean
    IsChanged = bDirty
End Function

Property Get Color(nC As Integer) As Long
    If nC >= 0 And nC < objColor.NumPens Then
        Color = objColor.Color(nC)
    End If
End Property

Property Let Color(nC As Integer, nX As Long)
    If nC >= 0 And nC < objColor.NumPens Then
        objColor.Color(nC) = nX
    End If
End Property

Public Sub LoadColors(stFile As String)
    objColor.Load stFile
End Sub
Public Sub SaveColors(stFile As String)
    objColor.Save stFile
End Sub

Public Sub DeSelect()
Dim ii As Integer, jj As Integer
        For jj = 0 To 11
            For ii = 0 To 11
                arBSelected(ii, jj) = False
            Next ii
        Next jj
    bSelectOn = False
End Sub

Property Get Selected(nX As Integer, nY As Integer) As Boolean
Dim nRetval As Integer

    'If the requested square is inside the limits
    If (nX >= 0) And (nX < 12) And (nY >= 0) And (nY < 12) Then
        'Return the selected status
        nRetval = arBSelected(nX, nY)
    Else
        'Return the default value of 1
        nRetval = False
    End If

    Selected = nRetval

End Property

Property Let Selected(nX As Integer, nY As Integer, nC As Boolean)
    'If the requested square is inside the limits
    If (nX >= 0) And (nX < 12) And (nY >= 0) And (nY < 12) Then
        'Change the square of the grid
        arBSelected(nX, nY) = nC
        bSelectOn = True
    End If
    'Otherwise, ignore the call
End Property

Property Get Grid(nX As Integer, nY As Integer) As Integer
Dim nRetval As Integer

    'If the requested square is inside the limits
    If (nX >= 0) And (nX < 12) And (nY >= 0) And (nY < 12) Then
        'Return the square of the grid
        nRetval = arGrid(nX, nY).Value
    Else
        'Return the default value of 1
        nRetval = 1
    End If

    Grid = nRetval
End Property

Property Let Grid(nX As Integer, nY As Integer, nC As Integer)
    'If the requested square is inside the limits
    If (nX >= 0) And (nX < 12) And (nY >= 0) And (nY < 12) Then
        'Change the square of the grid
        arGrid(nX, nY).Value = nC
        bDirty = True
    End If
    'Otherwise, ignore the call
End Property


Public Sub Init()
    Clear 0
    objColor.Init
End Sub


Public Sub Clear(nFaceKey As Integer)
Dim ii As Integer, jj As Integer
    'Setting the color key to 0 or 25+ will result
    'in a test pattern
    If (nFaceKey < 1) Or (nFaceKey > 24) Then
        For jj = 0 To 11
            For ii = 0 To 11
                arGrid(ii, jj).Value = 1
                If jj = 2 Then arGrid(ii, jj).Value = 5
                If jj = 3 Then arGrid(ii, jj).Value = 9
                If jj = 4 Then arGrid(ii, jj).Value = 13
                If jj = 5 Then arGrid(ii, jj).Value = 17
                If jj = 6 Then arGrid(ii, jj).Value = 21
            Next ii
        Next jj
    'Otherwise, clear all blocks to the requested face
    Else
        For jj = 0 To 11
            For ii = 0 To 11
                arGrid(ii, jj).Value = nFaceKey
            Next ii
        Next jj
    End If
    
    DeSelect
End Sub

Public Sub FlipGrid()
Dim ii As Integer, jj As Integer
Dim arTempGrid(12, 12) As New clsRBlock
         
    For jj = 0 To 11
        For ii = 0 To 11
            'place every block into its new location
            'in the temporary grid
                arTempGrid(11 - ii, jj).Value = arGrid(ii, jj).Value
        Next ii
        
        For ii = 0 To 11
            'place every block from the temporary grid
            'back into the normal grid
                arGrid(ii, jj).Value = arTempGrid(ii, jj).Value
        Next ii
    
    Next jj
       
    'Note that mathematically, even if the pattern is rotated,
    'it's still the same pattern and therefore doesn't need to
    'be saved.  No block has changed with respect to its neighbors.
    'However, this is not mathematics class
    bDirty = True

End Sub

Public Sub Roll(nX As Integer, nY As Integer, nSide As String)
    Select Case nSide
        Case "Left":    arGrid(nX, nY).Value = arGrid(nX, nY).LeftBlock
        Case "Right":   arGrid(nX, nY).Value = arGrid(nX, nY).RightBlock
        Case "Up":      arGrid(nX, nY).Value = arGrid(nX, nY).TopBlock
        Case "Down":    arGrid(nX, nY).Value = arGrid(nX, nY).BottomBlock
        Case "Around":  arGrid(nX, nY).Value = arGrid(nX, nY).OppositeBlock
        Case "Over":    arGrid(nX, nY).Value = arGrid(nX, nY).OverBlock
    End Select
    'Note that mathematically, even if the pattern is rotated,
    'it's still the same pattern and therefore doesn't need to
    'be saved.  No block has changed with respect to its neighbors.
    'However, this is not mathematics class
    bDirty = True
End Sub

Public Sub Rotate(nTimes As Integer)
Dim ii As Integer, jj As Integer, nloop As Integer
Dim arTempGrid(12, 12) As New clsRBlock, nValue As Integer

    For nloop = 1 To nTimes

    For jj = 0 To 11
        For ii = 0 To 11
            'place every block into its new location
            'in the temporary grid
                nValue = arGrid(ii, jj).Rotate
                arTempGrid(11 - jj, ii).Value = nValue
        Next ii
    Next jj
    
    For jj = 0 To 11
        For ii = 0 To 11
            'place every block from the temporary grid
            'back into the normal grid
                arGrid(ii, jj).Value = arTempGrid(ii, jj).Value
        Next ii
    Next jj
    
    Next nloop
    'Note that mathematically, even if the pattern is rotated,
    'it's still the same pattern and therefore doesn't need to
    'be saved.  No block has changed with respect to its neighbors.
    'However, this is not mathematics class
    bDirty = True

End Sub

Public Function Load(stFileName As String)
Dim ii As Integer, jj As Integer, nFile As Integer
Dim stLine As String
    
    Me.Clear 1
    
    If stFileName = "" Then GoTo EndSub
    nFile = FreeFile
    
    Open stFileName For Input As nFile
       
    'Load File
    Line Input #nFile, stLine
    Line Input #nFile, stLine
    
    For jj = 0 To 11
    Line Input #nFile, stLine
        For ii = 0 To 11
            arGrid(ii, jj).Value = Decode(Left(stLine, 1))
            If Len(stLine) > 0 Then stLine = Right(stLine, Len(stLine) - 1)
        Next ii
    Next jj
               
    Close #nFile
    
    bDirty = False
EndSub:
End Function

Public Function Save(stFileName As String)
Dim ii As Integer, jj As Integer, nFile As Integer
Dim stLine As String
       
    If stFileName = "" Then GoTo EndSub
    nFile = FreeFile
    
    Open stFileName For Output As nFile
    
     Print #nFile, "12"
     Print #nFile, "12"
          
    'Save File
    For jj = 0 To 11
        stLine = ""
        For ii = 0 To 11
            stLine = stLine & Encode(arGrid(ii, jj).Value)
        Next ii
        Print #nFile, stLine
    Next jj
            
    
    Close #nFile
    
    bDirty = False
EndSub:
End Function

Private Function Decode(stLetter As String) As Integer
Dim nRetval As Integer
    Select Case (stLetter)
        Case "a": nRetval = 1
        Case "b": nRetval = 2
        Case "c": nRetval = 3
        Case "d": nRetval = 4
        Case "e": nRetval = 5
        Case "f": nRetval = 6
        Case "g": nRetval = 7
        Case "h": nRetval = 8
        Case "i": nRetval = 9
        Case "j": nRetval = 10
        Case "k": nRetval = 11
        Case "l": nRetval = 12
        Case "m": nRetval = 13
        Case "n": nRetval = 14
        Case "o": nRetval = 15
        Case "p": nRetval = 16
        Case "q": nRetval = 17
        Case "r": nRetval = 18
        Case "s": nRetval = 19
        Case "t": nRetval = 20
        Case "u": nRetval = 21
        Case "v": nRetval = 22
        Case "w": nRetval = 23
        Case "x": nRetval = 24
        Case Else: nRetval = 1
    End Select
    Decode = nRetval
End Function

Private Function Encode(nNumber As Integer) As String
Dim nRetval As String
    Select Case (nNumber)
        Case 1: nRetval = "a"
        Case 2: nRetval = "b"
        Case 3: nRetval = "c"
        Case 4: nRetval = "d"
        Case 5: nRetval = "e"
        Case 6: nRetval = "f"
        Case 7: nRetval = "g"
        Case 8: nRetval = "h"
        Case 9: nRetval = "i"
        Case 10: nRetval = "j"
        Case 11: nRetval = "k"
        Case 12: nRetval = "l"
        Case 13: nRetval = "m"
        Case 14: nRetval = "n"
        Case 15: nRetval = "o"
        Case 16: nRetval = "p"
        Case 17: nRetval = "q"
        Case 18: nRetval = "r"
        Case 19: nRetval = "s"
        Case 20: nRetval = "t"
        Case 21: nRetval = "u"
        Case 22: nRetval = "v"
        Case 23: nRetval = "w"
        Case 24: nRetval = "x"
        Case Else: nRetval = "a"
    End Select
    Encode = nRetval
End Function

Public Function DrawGrid(picX As PictureBox, nSize As Integer, nSpacer As Integer)
Dim ii As Integer, jj As Integer

For jj = 0 To Width - 1
    For ii = 0 To Width - 1
        'Draw each individual block
    Next ii
Next jj
End Function
